@@ -25,7 +25,7 @@
 #include <bonobo/bonobo-generic-factory.h>
 #include <bonobo/bonobo-context.h>
 #include <bonobo/bonobo-running-context.h>
-
+#include <bonobo/bonobo-debug.h>
 #include <bonobo/bonobo-types.h>
 
 #define DEFAULT_LAST_UNREF_TIMEOUT 	2000
@@ -137,6 +137,21 @@ bonobo_generic_factory_construct (BonoboGenericFactory   *factory,
 
 	if (ret != Bonobo_ACTIVATION_REG_SUCCESS) {
 		bonobo_object_unref (BONOBO_OBJECT (factory));
+#ifdef G_ENABLE_DEBUG
+		if (_bonobo_debug_flags & BONOBO_DEBUG_LIFECYCLE) {
+			const char *err;
+			switch (ret)
+			{
+			case Bonobo_ACTIVATION_REG_SUCCESS:        err = "success";        break;
+			case Bonobo_ACTIVATION_REG_NOT_LISTED:     err = "not listed";     break;
+			case Bonobo_ACTIVATION_REG_ALREADY_ACTIVE: err = "already active"; break;
+			case Bonobo_ACTIVATION_REG_ERROR: 	   err = "error";          break;
+			default: err = "(invalid error!)"; break;
+			}
+			g_warning ("'%s' factory registration failed: %s",
+				   act_iid, err);
+		}
+#endif
 		return NULL;
 	}
 
@@ -283,14 +298,18 @@ BONOBO_TYPE_FUNC_FULL (BonoboGenericFactory,
 static gboolean
 startup_timeout (gpointer data)
 {
+	BonoboGenericFactory *factory = BONOBO_GENERIC_FACTORY (data);
 	bonobo_main_quit ();
+	factory->priv->startup_timeout_id = 0;
 	return FALSE;
 }
 
 static gboolean
 last_unref_timeout (gpointer data)
 {
+	BonoboGenericFactory *factory = BONOBO_GENERIC_FACTORY (data);
 	bonobo_main_quit ();
+	factory->priv->last_unref_timeout_id = 0;
 	return FALSE;
 }
 
